local function OnUnevenGroundDetected(inst, data)
    inst.components.des_carefulwalker_water:TrackTarget(data.inst, data.radius, data.period)
end

local des_carefulwalker_water = Class(function(self, inst)
    self.inst = inst
    self.careful = false
    self.carefulwalkingspeedmult = TUNING.CAREFUL_SPEED_MOD
    self.targets = {}
    inst:ListenForEvent("des_unevenwaterdetected", OnUnevenGroundDetected)
end)

function des_carefulwalker_water:OnRemoveFromEntity()
    inst:RemoveEventCallback("des_unevenwaterdetected", OnUnevenGroundDetected)
end

function des_carefulwalker_water:SetCarefulWalkingSpeedMultiplier(mult)
--[[
    mult = math.clamp(mult, 0, 1)
    if self.carefulwalkingspeedmult ~= mult then
        if next(self.targets) == nil then
            self.carefulwalkingspeedmult = mult
        elseif mult >= 1 then
            self.carefulwalkingspeedmult = 1
            self:ToggleCareful(false)
        elseif self.careful then
            self.carefulwalkingspeedmult = mult
            if self.inst.components.locomotor ~= nil then
                self.inst.components.locomotor:SetExternalSpeedMultiplier(self.inst, "carefulwalking", mult)
            end
        elseif self.carefulwalkingspeedmult >= 1 then
            self.carefulwalkingspeedmult = mult
            self:OnUpdate(0)
        else
            self.carefulwalkingspeedmult = mult
        end
    end--]]
end

function des_carefulwalker_water:TrackTarget(target, radius, duration)
    local data = self.targets[target]
    if data == nil then
        data = {}
        self.targets[target] = data
        self.inst:StartUpdatingComponent(self)
    end
    data.rangesq = radius * radius
    data.remaining = duration + .05
end

function des_carefulwalker_water:IsCarefulWalking()
    return self.careful
end

local water_update_time = 0.5
local water_per_sec = 1
local water_step = water_per_sec*water_update_time
local splash_offset = 0.33

function des_carefulwalker_water:ToggleCareful(careful)
    if careful then
        if not self.careful then
            self.careful = true
			
			
			self.inst.des_river_wet = self.inst:DoPeriodicTask(water_update_time, function()
				if self.inst.components.moisture then
					if self.inst.components.moisture:GetMoisturePercent()<0.25 then
						self.inst.components.moisture:DoDelta(water_step)
					end
					
					if self.inst.sg:HasStateTag("running") or self.inst.sg:HasStateTag("walking") then
						local x, y, z = self.inst.Transform:GetWorldPosition()

						x = x + math.random(-splash_offset,splash_offset)
						z = z + math.random(-splash_offset,splash_offset)
						
						local raindrop = SpawnPrefab("des_raindrop")
						raindrop.Transform:SetPosition(x, 0, z)
					end
				end
			end)
			
            --self.inst:PushEvent("carefulwalking", { careful = true })
        end
    elseif self.careful then
        self.careful = false
		self.inst.des_river_wet:Cancel()
		
		
        --self.inst:PushEvent("carefulwalking", { careful = false })
    end
end

function des_carefulwalker_water:OnUpdate(dt)
    local x, y, z = self.inst.Transform:GetWorldPosition()
    local checkcareful = (not self.inst:HasTag("playerghost")) and self.inst.components.locomotor ~= nil and self.inst.components.locomotor:FasterOnRoad()
    local careful = false
    local toremove
    for k, v in pairs(self.targets) do
        if v.remaining > dt and k:IsValid() then
            v.remaining = v.remaining - dt
            if checkcareful and k:GetDistanceSqToPoint(x, y, z) < v.rangesq then
                careful = true
                checkcareful = false
            end
        elseif toremove ~= nil then
            table.insert(toremove, k)
        else
            toremove = { k }
        end
    end

    if toremove ~= nil then
        for i, v in ipairs(toremove) do
            self.targets[v] = nil
        end
        if next(self.targets) == nil then
            self.inst:StopUpdatingComponent(self)
        end
    end

    self:ToggleCareful(careful)
end

return des_carefulwalker_water
