local Chakra = Class(function(self, inst)
    self.inst = inst

    if TheWorld.ismastersim then
        self.classified = inst.player_classified
    elseif self.classified == nil and inst.player_classified ~= nil then
        self:AttachClassified(inst.player_classified)
    end
end)

--------------------------------------------------------------------------

function Chakra:OnRemoveFromEntity()
    if self.classified ~= nil then
        if TheWorld.ismastersim then
            self.classified = nil
        else
            self.inst:RemoveEventCallback("onremove", self.ondetachclassified, self.classified)
            self:DetachClassified()
        end
    end
end

Chakra.OnRemoveEntity = Chakra.OnRemoveFromEntity

function Chakra:AttachClassified(classified)
    self.classified = classified
    self.ondetachclassified = function() self:DetachClassified() end
    self.inst:ListenForEvent("onremove", self.ondetachclassified, classified)
end

function Chakra:DetachClassified()
    self.classified = nil
    self.ondetachclassified = nil
end

--------------------------------------------------------

local function Set( netvar, value )
	if netvar then
		netvar:set(value)
	end
end

function Chakra:SetCurrent(current)
    Set(self.inst.currentthirst, current)
end

function Chakra:SetMax(maxthirst)
    Set(self.inst.maxthirst, maxthirst)
end

function Chakra:SetPenalty(penalty)
    Set(self.inst.penaltythirst, penalty)
end

function Chakra:Max()
    if self.inst.components.thirst ~= nil then
        return self.inst.components.thirst.max
    else
        return self.inst.maxthirst:value()
    end
end

function Chakra:GetPercent()
    if self.inst.components.thirst ~= nil then
        return self.inst.components.thirst:GetPercent()
    else
        return self.inst.currentthirst:value() / self.inst.maxthirst:value()
    end
end

function Chakra:GetCurrent()
    if self.inst.components.thirst ~= nil then
        return self.inst.components.thirst.current
    else 
        return self.inst.currentthirst:value()
    end
end

function Chakra:GetPenalty()
    if self.inst.components.thirst ~= nil then
        return self.inst.components.thirst.penalty
    else 
        return self.inst.penaltythirst:value()
    end
end

function Chakra:GetMaxWithPenalty()
    if self.inst.components.thirst ~= nil then
        return self.inst.components.thirst:GetMaxWithPenalty()
    else 
        return self.inst.maxthirst:value() - self.inst.maxthirst:value() * self.inst.penaltythirst:value()
    end
end

function Chakra:GetPercentWithPenalty()
	if self.inst.components.thirst ~= nil then
		return self.inst.components.thirst:GetPercentWithPenalty()
	else
		return self.inst.currentthirst:value() / (self.inst.maxthirst:value() - self.inst.maxthirst:value() * self.inst.penaltythirst:value())
	end
end

function Chakra:IsPartial()
    if self.inst.components.thirst ~= nil then
        return self.inst.components.thirst:IsPartial()
    elseif self.classified ~= nil then
        return self.inst.currentthirst:value() < self.inst.maxthirst:value()
    else
        return false
    end
end

return Chakra
