local assets =
{
    Asset("ANIM", "anim/des_chai_tree.zip"),
   -- Asset("ANIM", "anim/sapling_diseased_build.zip"),
    Asset("SOUND", "sound/common.fsb"),
}
local ivan_assets =
{
	Asset("ANIM", "anim/des_ivan_chai_tree.zip"),
   -- Asset("ANIM", "anim/sapling_diseased_build.zip"),
    Asset("SOUND", "sound/common.fsb"),
}
local prefabs =
{
    "des_chai_leaf_green",
    "dug_sapling",
    -- "disease_puff",
    -- "diseaseflies",
    "spoiled_food",
}

local function SpawnDiseasePuff(inst)
    SpawnPrefab("disease_puff").Transform:SetPosition(inst.Transform:GetWorldPosition())
end

local function SetDiseaseBuild(inst)
    inst.AnimState:SetBuild("sapling_diseased_build")
end

local function ondiseasedfn(inst)
    inst.components.pickable:ChangeProduct("spoiled_food")
    if POPULATING then
        SetDiseaseBuild(inst)
    else
        if inst.components.pickable:CanBePicked() then
            inst.AnimState:PlayAnimation("transform")
            inst.AnimState:PushAnimation("sway", true)
        elseif inst.components.witherable ~= nil
            and inst.components.witherable:IsWithered()
            or inst.components.pickable:IsBarren() then
            inst.AnimState:PlayAnimation("transform_dead")
            inst.AnimState:PushAnimation("idle_dead", false)
        else
            inst.AnimState:PlayAnimation("transform_empty")
            inst.AnimState:PushAnimation("empty", false)
        end
        inst:DoTaskInTime(6 * FRAMES, SpawnDiseasePuff)
        inst:DoTaskInTime(10 * FRAMES, SetDiseaseBuild)
    end
end

local function makediseaseable(inst)
    if inst.components.diseaseable == nil then
        inst:AddComponent("diseaseable")
        inst.components.diseaseable:SetDiseasedFn(ondiseasedfn)
    end
end

local function ontransplantfn(inst)
    inst.components.pickable:MakeEmpty()
    makediseaseable(inst)
    inst.components.diseaseable:RestartNearbySpread()
end

local function dig_up(inst, worker)
    if inst.components.pickable ~= nil and inst.components.lootdropper ~= nil then
        local withered = inst.components.witherable ~= nil and inst.components.witherable:IsWithered()
        local diseased = inst.components.diseaseable ~= nil and inst.components.diseaseable:IsDiseased()

        if diseased then
            SpawnDiseasePuff(inst)
        elseif inst.components.diseaseable ~= nil and inst.components.diseaseable:IsBecomingDiseased() then
            SpawnDiseasePuff(inst)
            if worker ~= nil then
                worker:PushEvent("digdiseasing")
            end
        end

        if inst.components.pickable:CanBePicked() then
            inst.components.lootdropper:SpawnLootPrefab(inst.components.pickable.product)
        end

        inst.components.lootdropper:SpawnLootPrefab(
            (withered or diseased) and
            "twigs" or
            "dug_sapling"
        )
    end
    inst:Remove()
end



local function FindRespawn(inst, dist, atm, theta_delta)
	local attemp = atm or 0
    local dst = dist or 4
	local radius = dst
	local theta = theta_delta or math.random()*2*PI
	local offset = Vector3(radius * math.cos( theta ), 0, -radius * math.sin( theta ) )
	local pt = Vector3(inst.Transform:GetWorldPosition()) + offset  
	
	
	if pt and TheWorld.Map:GetTileAtPoint(pt:Get()) ~= GROUND.IMPASSABLE then
		return pt
    elseif attemp<20 then
		if theta_delta then
			FindRespawn(inst, dst+0.75, attemp+1)
		else
			FindRespawn(inst, dst+0.75, attemp+1, theta+PI)			
		end
	else
		return
	end
	
end

local function onpickedfn(inst, picker)
    inst.AnimState:PlayAnimation("rustle")
    inst.AnimState:PushAnimation("picked", false)
    if inst.components.diseaseable ~= nil then
        if inst.components.diseaseable:IsDiseased() then
            SpawnDiseasePuff(inst)
        elseif inst.components.diseaseable:IsBecomingDiseased() then
            SpawnDiseasePuff(inst)
            if picker ~= nil then
                picker:PushEvent("pickdiseasing")
            end
        end
    end
	
	if inst.typename and inst.typename == "ivan" then
		local prefab = "des_ivan_chai_tree"
		
		----Check if we've got beehive nearby----
		local minrad=1.5
		local maxrad=4
		local pt = inst:GetPosition()
		local ents = TheSim:FindEntities(pt.x, pt.y, pt.z, maxrad)
		local count=0
		for k,v in pairs(ents) do
			if v.prefab == "beehive" or v.prefab =="wasphive" then
				count = count + 1
			end
		end
		if count>=1 then return end
		-----------------------------------------
		
		local offest = FindRespawn(inst, 19, 0, nil)

		if offest then
			--print("RESPAWN IVAN CHAI VALID LOCATION")
			local marsh_plant = SpawnPrefab("marsh_plant")
			marsh_plant.Transform:SetPosition(offest.x, 0 ,offest.z)
			local color = 0.1
			marsh_plant.persists = false
			marsh_plant.AnimState:SetMultColour(color, color, color, color)
			marsh_plant:AddTag("NOCLICK")
			marsh_plant:DoTaskInTime(TUNING.DES_THIRST_IVAN_CHAI_RESPAWN*0.8+math.random()*0.4*TUNING.DES_THIRST_IVAN_CHAI_RESPAWN, function()
				local newplant = SpawnPrefab(prefab)
				newplant.Transform:SetPosition(offest.x, 0 ,offest.z)
				marsh_plant:Remove()
				--rint("RESPAWN IVAN CHAI")
			end)
		end
		inst:DoTaskInTime(1, function()
			inst:Remove()
		end)	
	end
end

local function onregenfn(inst)
    inst.AnimState:PlayAnimation("grow")
    inst.AnimState:PushAnimation("sway", true)
end

local function makeemptyfn(inst)
    if not POPULATING and
        (   inst.components.witherable ~= nil and
            inst.components.witherable:IsWithered() or
            inst.AnimState:IsCurrentAnimation("idle_dead")
        ) then
        inst.AnimState:PlayAnimation("dead_to_empty")
        inst.AnimState:PushAnimation("empty", false)
    else
        inst.AnimState:PlayAnimation("empty")
    end
end

local function makebarrenfn(inst, wasempty)
    if not POPULATING and
        (   inst.components.witherable ~= nil and
            inst.components.witherable:IsWithered()
        ) then
        inst.AnimState:PlayAnimation(wasempty and "empty_to_dead" or "full_to_dead")
        inst.AnimState:PushAnimation("idle_dead", false)
    else
        inst.AnimState:PlayAnimation("idle_dead")
    end
end

local function OnPreLoad(inst, data)
    if data ~= nil and (data.pickable ~= nil and data.pickable.transplanted or data.diseaseable ~= nil) then
        makediseaseable(inst)
    end
end


local function buildteaTree(typename)

local function fn()
    local inst = CreateEntity()

    inst.entity:AddTransform()
    inst.entity:AddAnimState()
    inst.entity:AddMiniMapEntity()
    inst.entity:AddNetwork()

    inst.MiniMapEntity:SetIcon("des_chai_tree.png")

	inst.typename = typename
	
    inst.AnimState:SetRayTestOnBB(true)
	if typename == "green" then
		inst.AnimState:SetBank("des_chai_tree")
		inst.AnimState:SetBuild("des_chai_tree")
		
		
		local s = 0.17+math.random()*0.05
		inst.Transform:SetScale(s,s,s)
		local color = 0.75 + math.random() * 0.25
		inst.AnimState:SetMultColour(color, color, color, 1)
	else
		inst.AnimState:SetBank("des_ivan_chai_tree")
		inst.AnimState:SetBuild("des_ivan_chai_tree")
		
		local s = 1.1 +math.random()*0.3
		inst.Transform:SetScale(s,s,s)
		local color = 0.9 + math.random() * 0.1
		inst.AnimState:SetMultColour(color, color, color, 1)
	end
    inst.AnimState:PlayAnimation("sway", true)

    inst:AddTag("renewable")

    --witherable (from witherable component) added to pristine state for optimization
    inst:AddTag("witherable")

	
    inst.entity:SetPristine()

    if not TheWorld.ismastersim then
        return inst
    end

    inst.AnimState:SetTime(math.random() * 2)

    inst:AddComponent("pickable")
    inst.components.pickable.picksound = "dontstarve/wilson/pickup_reeds"



	if typename == "green" then
		inst:AddComponent("witherable")
		inst.components.pickable:SetUp("des_chai_leaf_green", TUNING.DES_THIRST_CHAITREE_REGROW_TIME, TUNING.DES_THIRST_CHAITREE_NUMBER)
	elseif typename == "ivan" then
		inst.components.pickable:SetUp("des_chai_leaf_ivan", TUNING.DES_THIRST_CHAITREE_REGROW_TIME, TUNING.DES_THIRST_CHAITREE_NUMBER)
	end



    inst.components.pickable.onregenfn = onregenfn
    inst.components.pickable.onpickedfn = onpickedfn
    inst.components.pickable.makeemptyfn = makeemptyfn
    inst.components.pickable.ontransplantfn = ontransplantfn
    inst.components.pickable.makebarrenfn = makebarrenfn


	if typename == "green" then
		inst:AddComponent("witherable")
	---elseif typename == "ivan" then

	end





    inst:AddComponent("inspectable")

    inst:AddComponent("lootdropper")

    if not GetGameModeProperty("disable_transplanting") then
     --[[   inst:AddComponent("workable")
        inst.components.workable:SetWorkAction(ACTIONS.DIG)
        inst.components.workable:SetOnFinishCallback(dig_up)
        inst.components.workable:SetWorkLeft(1)--]]
    end

    MakeMediumBurnable(inst)
    MakeSmallPropagator(inst)
    MakeNoGrowInWinter(inst)
    MakeHauntableIgnite(inst)
    ---------------------

    inst.OnPreLoad = OnPreLoad
    --inst.MakeDiseaseable = makediseaseable



    return inst
end

if typename == "green" then
	return Prefab("des_chai_tree", fn, assets, prefabs)
else
	return Prefab("des_ivan_chai_tree", fn, ivan_assets, prefabs)
end
end

return buildteaTree("green"),
buildteaTree("ivan")
