require "prefabutil"
require "recipe"
require "modutil"

local assets=
{
	Asset("ANIM", "anim/des_water_barrel.zip"),
    Asset("ATLAS", "images/inventoryimages/des_water_barrel.xml"),
    Asset("IMAGE", "images/inventoryimages/des_water_barrel.tex"),
	Asset("SOUNDPACKAGE", "sound/pickle_barrel.fev"),
	Asset("SOUND", "sound/pickle_barrel_bank.fsb"),
	Asset("SOUND", "sound/monkey.fsb"),

	Asset("ANIM", "anim/des_ui_wbarrel1x3.zip"),
}

local prefabs = {
   "monkey",

}


local function shake(inst)
    inst.AnimState:PlayAnimation(math.random() > .5 and "move1" or "move2")
    inst.AnimState:PushAnimation("idle_empty")
    inst.SoundEmitter:PlaySound("dontstarve/creatures/monkey/barrel_rattle")
end




--  Break the pickle barrel when it is hammered upon
local function onhammered(inst, worker)
	if inst.frozen == true then
		inst:RemoveComponent("lootdropper")	
		inst:AddComponent("lootdropper")
		inst.components.lootdropper:SetLoot({ "boards", "rope"})
		inst.components.lootdropper:DropLoot()
		local insert_ice = math.floor(inst.components.cwater.current/100)+1
		inst.components.lootdropper:SetLoot({"ice"})

		local i = 0
		repeat
		i = i + 1
			inst.components.lootdropper:DropLoot()
		until i > insert_ice

	end


	SpawnPrefab("collapse_small").Transform:SetPosition(inst.Transform:GetWorldPosition())
	inst.SoundEmitter:PlaySound("dontstarve/common/destroy_wood")
	inst:Remove()
end

local function FakeDrop(inst)
	--if true==true then
	--return
	--end
	
	local found = inst.components.container:FindItem(function(find_fakedrop) return find_fakedrop==inst.des_fakedrop end)
	if found then
		return
	elseif not found and inst.des_fakedrop then
		inst.des_fakedrop:Remove()
	end
	

	
	if not found then
		inst.des_fakedrop = SpawnPrefab("des_waterdrop_fake")
		
		inst.des_fakedrop.original_owner = inst
		print("SPAWNED FAKE DROP")
		inst.components.container:GiveItem(inst.des_fakedrop, inst.fakedrop_slot)
	end
end




local function onopen(inst)
	FakeDrop(inst)

	inst:AddTag("des_watercatcher_update_fakedrop")
	inst.components.cwater:GetDiscCatcher()
	inst.SoundEmitter:PlaySound("pickle_barrel/pickle_barrel/open", "open")
	
	--inst.AnimState:PlayAnimation("open")
end

local function onclose(inst)
	inst.SoundEmitter:PlaySound("pickle_barrel/pickle_barrel/close", "close")

	FakeDrop(inst)
	inst:RemoveTag("des_watercatcher_update_fakedrop")
end




local function onbuilt(inst)
	inst.AnimState:PlayAnimation("build")
	inst.AnimState:PushAnimation("idle_empty", false)
end


local function ongetitem(inst)
	FixAnim(inst)
	inst:DoTaskInTime(0.1, function()
		FakeDrop(inst)
	end)
	
	local player = nil
	if TheSim:GetGameID() == "DST" then
	    for _, playerf in pairs(AllPlayers) do
	        if (not playerf:HasTag("playerghost")) and (not playerf:HasTag("ghost")) and inst:GetDistanceSqToInst(playerf)<6*6 then
				player = playerf
            end
	    end
	end		
--	if inst:HasTag("des_transfuse_to_2nd_slot") then 
	--if inst:HasTag("water_trap_or_dest") then
		local bottle1 = inst.components.container:GetItemInSlot(1)
		local bottle2 = inst.components.container:GetItemInSlot(3)

		
	if bottle1 and bottle1.prefab == "bucket" then
		bottle1:Remove()
		local newbucket = nil
		if inst.components.cwater.current>50 then
			newbucket = SpawnPrefab("bucketfull")
			inst.components.cwater:DoDelta(-50)
		else
			newbucket = SpawnPrefab("bucket")
		end
		inst.components.container:GiveItem(newbucket, nil, nil, true)
		inst.components.container:DropItem(newbucket)
		inst.components.container:Close()
		return
	end
	if bottle1 then
		if bottle1:HasTag("des_well_poisoner") then
			local word = TUNING.THIRST_POISON_PHRASES[math.random(#TUNING.THIRST_POISON_PHRASES)]
			if player then
				player.components.talker:Say(word, 4, false)
			end
			inst.components.cwater:Poison()
			--UseAndDrop(ammount, maximum, bucket, player)
			inst.components.des_easy_code:UseAndDrop(1, 1, bottle1, player)
			
			return
		end	
		if bottle1:HasTag("des_well_watercleaner") then
			local word = TUNING.THIRST_CLEARPOISON_PHRASES[math.random(#TUNING.THIRST_CLEARPOISON_PHRASES)]
			if player then
				player.components.talker:Say(word, 4, false)
			end
			
			inst.components.cwater:ClearPoison()

			--UseAndDrop(ammount, maximum, bucket, player)
			inst.components.des_easy_code:UseAndDrop(1, 1, bottle1, player)
			
			return
		end		
		
		if bottle1:HasTag("des_water_saltremoveer") then
			local word = TUNING.THIRST_CLEARSALT_PHRASES[math.random(#TUNING.THIRST_CLEARSALT_PHRASES)]
			if player then
				player.components.talker:Say(word, 4, false)
			end
			
			inst.components.cwater:ClearSalt()

			--UseAndDrop(ammount, maximum, bucket, player)
			inst.components.des_easy_code:UseAndDrop(1, 1, bottle1, player)
			
			return
		end		
	end
		
		
		
		
		
		
		
		
		if inst.components.container:GetItemInSlot(inst.fakedrop_slot+1) and inst.components.container:GetItemInSlot(inst.fakedrop_slot+1):HasTag("des_drop_fromcatcher") then 
			inst.components.container:DropItem(inst.components.container:GetItemInSlot(inst.fakedrop_slot+1))
			if inst.components.container:IsOpen() == true then
				inst.components.container:Close()
			end
			if player then player.components.talker:Say("It doesn't fit here.") end
		end
		
		if bottle1 and bottle1:HasTag("des_fillable_on_catcher") then
			bottle1.components.cwater:Transfuse(inst, bottle1, nil, true, true)
			--if inst.components.container:IsOpen() == false and (not player) then
			inst.components.container:DropItem(bottle1)
			--end
		end

		if bottle2 and bottle2:HasTag("des_fillable_on_catcher") then
			inst.components.cwater:Transfuse(bottle2, inst, nil, true, true)
			--if inst.components.container:IsOpen() == false then
			inst.components.container:DropItem(bottle2)
			--end
		end

		
		inst.components.container:Close()
--	inst.components.cwater:AddBottle(inst)
---end
end

local function onloseitem(inst)
	FakeDrop(inst)
end

local function onhit(inst, worker)
	if inst.components.cwater.current>50 and inst.frozen == false then
		local splash = SpawnPrefab("splash_ocean")
		splash.Transform:SetPosition(inst.Transform:GetWorldPosition())
		inst.components.cwater:GroundPour(nil, nil, 400)
	end
	inst.AnimState:PlayAnimation(inst.hit_anim)
    inst.AnimState:PushAnimation(inst.idle_anim)
end


local function OnFinishedDest(inst)
	SpawnPrefab("boards").Transform:SetPosition(inst.Transform:GetWorldPosition())
	SpawnPrefab("rope").Transform:SetPosition(inst.Transform:GetWorldPosition())
	SpawnPrefab("cutgrass").Transform:SetPosition(inst.Transform:GetWorldPosition())
	inst:Remove()
end



local function OnIsDay(inst, isday)
    if isday or (inst.components.cwater.current>10) or inst.frozen then
        inst.components.childspawner:StopSpawning()
    elseif inst.components.cwater.current <=10 then
        inst.components.childspawner:StartSpawning()
    end
end


local function OnSnowLevel(inst, snowlevel)

    if snowlevel > .02 then
        if not inst.frozen then
            inst.frozen = true
            inst.SoundEmitter:PlaySound("dontstarve/winter/pondfreeze")
            inst.components.childspawner:StopSpawning()
			inst.components.container.canbeopened = false
			----how de fuck this works?
			
        end
    elseif inst.frozen then
        inst.frozen = false
		inst.components.container.canbeopened = true

    elseif inst.frozen == nil then
        inst.frozen = false
		inst.components.container.canbeopened = true
	end
	FixAnim(inst)
end

local function OnInit(inst)
	inst.task = nil
    inst:WatchWorldState("snowlevel", OnSnowLevel)
    OnSnowLevel(inst, TheWorld.state.snowlevel)
end


function FixAnim(inst)
	if inst.frozen == false then
		if inst.components.cwater:GetPercent() > 0.65 and inst.components.cwater:GetBadPercent() > 0.8  then
			inst.idle_anim = "idle_poison"
			inst.hit_anim = "hit_poison"
		elseif inst.components.cwater:GetPercent() > 0.65 then
			inst.idle_anim = "idle_full"
			inst.hit_anim = "hit_full"
			
		else
			inst.idle_anim = "idle_empty"
			inst.hit_anim = "hit_empty"
		end
	else
		if inst.components.cwater:GetPercent() < 0.65 then
			inst.idle_anim = "idle_empty_frozen"
			inst.hit_anim = "hit_empty_frozen"			
		else
			inst.idle_anim = "idle_full_frozen"
			inst.hit_anim = "hit_full_frozen"		
		end
	end
	inst.AnimState:PlayAnimation(inst.idle_anim)
end


local function fn(Sim)
	local inst = CreateEntity()
	inst.entity:AddTransform()
	local anim = inst.entity:AddAnimState()
	inst.entity:AddSoundEmitter()
    inst.entity:AddNetwork()
	
	local minimap = inst.entity:AddMiniMapEntity()
	minimap:SetIcon( "des_water_barrel.tex" )
    
    inst:AddTag("structure")
	
	inst:AddTag("water_dest_barrel")

	inst:AddTag("des_water_spoiling")
	
    MakeObstaclePhysics(inst, .5)
    
    inst.AnimState:SetBank("des_water_barrel")
    inst.AnimState:SetBuild("des_water_barrel")
    inst.AnimState:PlayAnimation("idle_empty")

    if not TheWorld.ismastersim then
        return inst
    end
	
	inst.entity:SetPristine()

	inst:AddComponent("cwater")
	inst.components.cwater:SetMax(TUNING.DES_WATER_BARREL_SIZE)
	inst.components.cwater:DoDelta(-TUNING.DES_WATER_BARREL_SIZE)
	--inst.components.cwater.watergainspeed = TUNING.WATERTRAP_GAIN_SPEED_GRASS
	inst:AddTag("water_gain_rain")

	inst.components.cwater.cleanspeed = TUNING.WATER_SPOIL_SPEED
		----badness per second per 100 water inside of container
	inst.components.cwater.spoilspeed = TUNING.DES_WATER_SPOIL_BARREL

	
	
    inst:AddComponent("container")
	inst.components.container:WidgetSetup("des_water_barrel")
    inst.components.container.onopenfn = onopen
    inst.components.container.onclosefn = onclose
	inst.widgetsetup = "des_water_barrel"
	inst.fakedrop_slot = 2
	inst:ListenForEvent("itemget", ongetitem)
    inst:ListenForEvent("itemlose", onloseitem)

    inst:AddComponent("inspectable")

	
	
	inst:AddComponent("finiteuses")
	inst.components.finiteuses:SetMaxUses(900)
	inst.components.finiteuses:SetUses(900)
	inst.components.finiteuses:SetOnFinished(OnFinishedDest)
	
	
	
    inst:AddComponent("childspawner")
    inst.components.childspawner.childname = "monkey"
    inst.components.childspawner:SetSpawnedFn(shake)
    inst.components.childspawner:SetGoHomeFn(shake)
    inst.components.childspawner:SetRegenPeriod(TUNING.TOTAL_DAY_TIME * 3)
    inst.components.childspawner:SetSpawnPeriod(10)
    inst.components.childspawner:SetMaxChildren(1)
	
	inst:WatchWorldState("isday", OnIsDay)

	
	inst:DoPeriodicTask(10, function()
		FixAnim(inst)
	end)

    inst.task = inst:DoTaskInTime(0, OnInit)
	
	inst:AddComponent("des_easy_code")

    inst:AddComponent("lootdropper")
    inst:AddComponent("workable")
    inst.components.workable:SetWorkAction(ACTIONS.HAMMER)
    inst.components.workable:SetWorkLeft(3)
	inst.components.workable:SetOnFinishCallback(onhammered)
	inst.components.workable:SetOnWorkCallback(onhit)

	--MakeSnowCovered(inst, .01)    
	inst:ListenForEvent("onbuilt", onbuilt)
    return inst
end


return Prefab( "common/des_water_barrel", fn, assets, prefabs),
		MakePlacer( "common/des_water_barrel_placer", "des_water_barrel", "des_water_barrel", "idle_empty" ) 
