local assets =
{
    Asset("ANIM", "anim/des_well_hole.zip"),
	
	Asset("ATLAS", "images/inventoryimages/des_wellhole.xml"),
    Asset("IMAGE", "images/inventoryimages/des_wellhole.tex"),
}

local prefabs =
{
    "ghost",
    "amulet",
    "redgem",
    "gears",
    "bluegem",
    "nightmarefuel",
}


local function onfinishcallback(inst, worker)
    inst.AnimState:PlayAnimation("dug")
    inst:RemoveComponent("workable")
end


local function GetStatus(inst)
    if not inst.components.workable then
        return "DUG"
    end
	return "UNDUG"
end

local function OnSave(inst, data)
    if inst.components.workable == nil then
        data.dug = true
    end
end

local function OnLoad(inst, data)
    if data ~= nil and data.dug or inst.components.workable == nil then
        inst:RemoveComponent("workable")
        inst.AnimState:PlayAnimation("dug")
    end
end




local function ReturnChildren(inst)
    for k, child in pairs(inst.components.childspawner.childrenoutside) do
        if child.components.homeseeker ~= nil then
            child.components.homeseeker:GoHome()
        end
        child:PushEvent("gohome")
    end
end

local function OnIsDay(inst, isday)
    if isday ~= inst.dayspawn then
        inst.components.childspawner:StopSpawning()
        ReturnChildren(inst)
    elseif not TheWorld.state.iswinter and inst.components.workable == nil then
        inst.components.childspawner:StartSpawning()
    end
end





local function findfountain_spawnpoint(caster)
    local ground = TheWorld
    local centers = {}
    for i, node in ipairs(ground.topology.nodes) do
        if ground.Map:IsPassableAtPoint(node.x, 0, node.y) then
            table.insert(centers, {x = node.x, z = node.y})
        end
    end
    if #centers > 0 then
        local pos = centers[math.random(#centers)]
        return pos.x, 0, pos.z
    else
        return caster:GetPosition()
    end
end




local function OnInit(inst)
	if TUNING.THIRST_ENABLE_POST_GEN == true then
		local ammount_of_wells = c_countprefabs("des_water_well", true)
		local ammount_of_holes = c_countprefabs("des_well_hole", true)
		print("=========AMMOUNT_OF_HOLES =  "..ammount_of_holes)
		print("=========DES_THIRST_WATER_HOLES_MAX =  "..TUNING.DES_THIRST_WATER_HOLES_MAX)
		
		if ammount_of_holes > TUNING.DES_THIRST_WATER_HOLES_MAX then
			local des_speical_dummy = SpawnPrefab("des_speical_dummy")
			des_speical_dummy.Transform:SetPosition(0,0,0)
			
			print("========= TOO many holes=======")
			
			local i = 0
			repeat
			i = i + 1
			
			local found_hole = c_find("des_well_hole", nil, des_speical_dummy)
			if found_hole then
				local second_found_hole = c_find("des_well_hole", 75, found_hole)
				if second_found_hole then 
					print("========= REMOVING CLOSE HOLE =======")
					second_found_hole:Remove()
				end
				if i > TUNING.DES_THIRST_WATER_HOLES_MAX then
					found_hole:Remove()
				end
			end

			
			
			until i > ammount_of_holes
		end

		ammount_of_holes = c_countprefabs("des_well_hole", true)
		 
		if ((ammount_of_wells+ammount_of_holes) < (TUNING.DES_THIRST_WATER_HOLES_MAX-1)) and (math.random()<0.8) then
			print("=========(ammount_of_wells+ammount_of_holes) < TUNING.DES_THIRST_WATER_HOLES_MAX - 1")
			print("=========(START SPAWNING HOLEs============")

		
			local k = 0
			repeat
			k = k + 1

			local x1, y1, z1 = findfountain_spawnpoint(inst)
			local new_des_well_hole = SpawnPrefab("des_well_hole")
			new_des_well_hole.Transform:SetPosition(x1, 0, z1)
			
			until k > (TUNING.DES_THIRST_WATER_HOLES_MAX-ammount_of_wells-ammount_of_holes-1)
			
			ammount_of_holes = c_countprefabs("des_well_hole", true)
			print("=========Finished spawning =  ammount_of_holes"..ammount_of_holes)

		end

--[[
    inst.task = nil
    inst:WatchWorldState("isday", OnIsDay)
   -- inst:WatchWorldState("snowlevel", OnSnowLevel)
    OnIsDay(inst, TheWorld.state.isday)
  --  OnSnowLevel(inst, TheWorld.state.snowlevel)--]]
  
	end
end








local function fn()
    local inst = CreateEntity()
    inst.entity:AddTransform()
    inst.entity:AddAnimState()
    inst.entity:AddSoundEmitter()
    inst.entity:AddMiniMapEntity()
    inst.entity:AddNetwork()

	MakeObstaclePhysics(inst, 1.0)

	
    inst.AnimState:SetBank("des_well_hole")
    inst.AnimState:SetBuild("des_well_hole")
    inst.AnimState:PlayAnimation("undug")

	
	local minimap = inst.entity:AddMiniMapEntity()
	minimap:SetIcon( "des_wellhole.tex" )	
	
	
	--inst.AnimState:SetOrientation(ANIM_ORIENTATION.OnGround)
    inst.AnimState:SetLayer(LAYER_BACKGROUND)
    inst.AnimState:SetSortOrder(3)
	
	local s = 0.95
	inst.Transform:SetScale(s, s, s)

    inst.entity:SetPristine()

    if not TheWorld.ismastersim then
        return inst
    end

	inst.dayspawn = false
	inst.frozen = nil
	


    	
--	inst:AddComponent("trader")
--	inst.components.trader:SetAcceptTest(ShouldAcceptItem)
--	inst.components.trader.onaccept = OnGetItemFromPlayer
	
	
    inst:AddComponent("inspectable")
    inst.components.inspectable.getstatus = GetStatus

    inst:AddComponent("workable")
    inst.components.workable:SetWorkAction(ACTIONS.DIG)
    inst.components.workable:SetWorkLeft(1)
    inst:AddComponent("lootdropper")

    inst.components.workable:SetOnFinishCallback(onfinishcallback)
	
	inst:AddComponent("childspawner")
    inst.components.childspawner:SetRegenPeriod(TUNING.POND_REGEN_TIME)
    inst.components.childspawner:SetSpawnPeriod(TUNING.POND_SPAWN_TIME)
    inst.components.childspawner:SetMaxChildren(math.random(3, 4))
    inst.components.childspawner:StartRegen()
	--inst:DoTaskInTime(1+math.random()*5, OnInit)
	inst.components.childspawner.childname = "mosquito"
   -- inst:DoTaskInTime(0, oninit)

    inst.OnSave = OnSave
    inst.OnLoad = OnLoad

    return inst
end

return Prefab("des_well_hole", fn, assets, prefabs)
