local assets =
{
	Asset("ANIM", "anim/marsh_tile.zip"),
	Asset("ANIM", "anim/splash.zip"),
}

local prefabs =
{
	"marsh_plant",

}



local function SpawnPlants(inst, plantname)

	if inst.decor then
		for i,item in ipairs(inst.decor) do
			item:Remove()
		end
	end
	inst.decor = {}

	local plant_offsets = {}

	for i=1,math.random(15,25) do
		local a = math.random()*math.pi*2
		local x = math.sin(a)*1.9+math.random()*0.3
		local z = math.cos(a)*2.1+math.random()*0.3
		table.insert(plant_offsets, {x,0,z})
	end

	for k, offset in pairs( plant_offsets ) do
		local plant = SpawnPrefab( plantname )
		plant.entity:SetParent( inst.entity )
		plant.Transform:SetPosition( offset[1], offset[2], offset[3] )
		table.insert( inst.decor, plant )
	end
end


local function OnSnowCoverChange(inst, thresh)
	thresh = thresh or .02
	local snow_cover = GetSeasonManager() and GetSeasonManager():GetSnowPercent() or 0

	if snow_cover > thresh and not inst.frozen then
		inst.frozen = true
		inst.AnimState:PlayAnimation("frozen")
		inst.SoundEmitter:PlaySound("dontstarve/winter/pondfreeze")
	  --  inst.components.childspawner:StopSpawning()
	--	inst.components.fishable:Freeze()

        inst.Physics:SetCollisionGroup(COLLISION.OBSTACLES)
        inst.Physics:ClearCollisionMask()
        inst.Physics:CollidesWith(COLLISION.WORLD)
        inst.Physics:CollidesWith(COLLISION.ITEMS)
        inst.components.cwater:DoDelta(1000)

		for i,item in ipairs(inst.decor) do
			item:Remove()
		end
		inst.decor = {}
	elseif snow_cover < thresh and inst.frozen then
		inst.frozen = false
		inst.AnimState:PlayAnimation("idle"..inst.pondtype)
	    --inst.components.childspawner:StartSpawning()
		--inst.components.fishable:Unfreeze()

		inst.Physics:SetCollisionGroup(COLLISION.OBSTACLES)
        inst.Physics:ClearCollisionMask()
        inst.Physics:CollidesWith(COLLISION.WORLD)
        inst.Physics:CollidesWith(COLLISION.ITEMS)
        inst.Physics:CollidesWith(COLLISION.CHARACTERS)

		SpawnPlants(inst, inst.planttype)
	end
end

local function onload(inst, data, newents)
	OnSnowCoverChange(inst)
end
--[[
local function OnActivate(inst)
if inst.frozen == false then
		inst.components.activatable.inactive = true
		
	if inst.components.cwater.current>5 then
		if GetPlayer().components.moisture then
			GetPlayer().components.moisture:DoDelta(math.random(5,10))
		end
	end
		
	if inst.components.cwater.current> TUNING.POND_WATER_RESTORE then	
        GetPlayer().components.thirst:DoDelta(TUNING.POND_WATER_RESTORE)
	elseif inst.components.cwater.current>5 then
        GetPlayer().components.thirst:DoDelta(inst.components.cwater.current)
	else
		GetPlayer().components.talker:Say("The pond is empty!")
	end
        inst.components.cwater:DoDelta(-TUNING.POND_WATER_RESTORE)
		
		GetPlayer().components.thirst.timetosay = 4
		if GetPlayer().components.moisture then
			GetPlayer().components.moisture:DoDelta(math.random(3,9))
		end
		GetPlayer().components.temperature.current =GetPlayer().components.temperature.current-5
end
end--]]


local function GetVerb(inst)
	return STRINGS.DRINKFROM
end


local function onhammered(inst, worker)
	inst.components.lootdropper:DropLoot()
	SpawnPrefab("collapse_small").Transform:SetPosition(inst.Transform:GetWorldPosition())
	
	inst:Remove()
end

local function onhit(inst, worker)
end


local function commonfn(pondtype)
	local inst = CreateEntity()
	local trans = inst.entity:AddTransform()
	local anim = inst.entity:AddAnimState()
    inst.entity:AddSoundEmitter()
    inst.pondtype = pondtype
    MakeObstaclePhysics( inst, 1.95)

    anim:SetBuild("marsh_tile")
    anim:SetBank("marsh_tile")
    anim:PlayAnimation("idle"..pondtype, true)
	anim:SetOrientation( ANIM_ORIENTATION.OnGround )
	anim:SetLayer( LAYER_BACKGROUND )
	anim:SetSortOrder( 3 )

	local minimap = inst.entity:AddMiniMapEntity()
	minimap:SetIcon( "pond"..pondtype..".png" )
	
	
	inst.frozen = false
	inst:AddComponent("cwater")
    inst.components.cwater:SetMax(TUNING.PONDBULD_WATER)
    inst.components.cwater:DoDelta(-TUNING.PONDBULD_WATER)
	
	
	
	
    inst:AddComponent("lootdropper")
    inst:AddComponent("workable")
    inst.components.workable:SetWorkAction(ACTIONS.HAMMER)
    inst.components.workable:SetWorkLeft(4)
	inst.components.workable:SetOnFinishCallback(onhammered)
	inst.components.workable:SetOnWorkCallback(onhit) 	
	
	
	
	
	inst:AddTag("water_sourse")
	inst:AddTag("water_sourse_limited")
	inst:AddTag("water_gain_rain")
	inst:AddTag("water_pond")
    inst:AddComponent("inspectable")
    inst.components.inspectable.nameoverride = "pond"

  	inst:DoPeriodicTask(10, function()
		local per = inst.components.cwater.current/inst.components.cwater.max
		
  	if per > 0.8 then
		inst.components.inspectable:SetDescription("I can't see the bottom!")
	elseif per > 0.6 then
		inst.components.inspectable:SetDescription("It's half full.")
	elseif per > 0.4 then
		inst.components.inspectable:SetDescription("It's half empty.")
	elseif per > 0.35 then
		inst.components.inspectable:SetDescription("There are some water in it.")
	elseif per > 0.2 then
		inst.components.inspectable:SetDescription("Water will return to the soil soon.")
	elseif per > 0.1 then
		inst.components.inspectable:SetDescription("It will be empty soon!")
	else
		inst.components.inspectable:SetDescription("There are more dirt then water in it.")
	end

	end)
  
	inst.OnLoad = onload

	return inst
end



local function pondcave()
	local inst = commonfn("_cave")
		inst.planttype = "pondbuildsign"
	SpawnPlants(inst, inst.planttype)

	--These spawn nothing at this time.
	return inst
end

return	  Prefab("marsh/objects/pondbuild", pondcave, assets, prefabs),
		MakePlacer("marsh/objects/pondbuild_placer", "marsh_tile", "marsh_tile", "idle_cave")
